# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Frame
from horus_media import Client, ImageRequestBuilder, ImageProvider, Mode, Scales, Direction, Grid

# This example shows how to request a panoramic image

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
frames = Frames(connection)
client = Client()
image_provider = ImageProvider()

# Get a frame
frame = Frame(frames.query())

if frame is None:
    print("No recordings!")
    exit()

# Set parameters
mode = Mode.panoramic
scale = Scales.Px_2048

# Get the image

request_builder = ImageRequestBuilder(frame.recordingid, frame.uuid)

#(min, max) yaw in deg
directions = {
    "front":(-45, 45),
    "back":(135, 225),
    "left":(-135, -45),
    "right":(45, 135),
}

grid = Grid()
for direction in directions:
    min_yaw = directions[direction][0]
    max_yaw = directions[direction][1]
    sections = grid.filter(h_min=-44, h_max=44, w_min=min_yaw, w_max=max_yaw)
    requests = client.fetch_all(request_builder.build(Mode.panoramic, scale, section) for section in sections)
    result = image_provider.combine(requests, scale.size, scale.size)

    # Save the file
    filename = ".\\panoramic_{}_{}.jpg".format(frame.index, direction)

    with open(filename, 'wb') as f:
        f.write(result.image.getvalue())
        result.image.close()



# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Recordings, Frame, Recording

# This example shows how to iterate over all the recordings

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
recordings = Recordings(connection)

cursor = recordings.all()

recording = Recording(cursor)

while recording is not None:
    print(" ", recording.id, " ", recording.directory)
    recording = Recording(cursor)



# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Frame
from horus_media import Client, ImageRequestBuilder, ImageProvider, Mode, Size, Geometry

# This example shows how to request a georeferenced orthographic image (geotiff)

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
frames = Frames(connection)
client = Client()
image_provider = ImageProvider()

# Get a frame
frame = Frame(frames.query())

if frame is None:
    print("No recordings!")
    exit()

# If available, the altitude of the next frame is used in calculating ground plane 
# (optional, required if results should be equal to the Horus MoviePlayer)
cursor = frames.query(recordingid=frame.recordingid, index=frame.index + 1)
next_frame = Frame(cursor)
if next_frame is not None:
    alti_next = next_frame.altitude
else:
    alti_next = None

# Set parameters

# Size of render buffer, should at least be the size of the original camera resolution of the roi
size = Size(4096, 2048) 

# The geometry values relative to the frame center, same as used in the Horus MoviePlayer ortho projection
geom_scale = 400 # output scale (px/m)
geom_width = 6 # geometry width (m)
geom_height = 2 # geometry height (m)
geom_dist = 4 # geometry distance (m)
geom_shift = 0 # geometry shift (m)
geometry = Geometry(geom_scale, geom_width, geom_height, geom_dist, geom_shift, alti_next)

# Get the image
request_builder = ImageRequestBuilder(frame.recordingid, frame.uuid)
request = client.fetch(request_builder.build_orthographic(size, geometry))
result = image_provider.fetch(request)

# Save the file
filename = ".\\orthographic_{}.tif".format(frame.index)

with open(filename, 'wb') as f:
    f.write(result.image.getvalue())
    result.image.close()



# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Recordings, Frame, Recording

# This example shows how to iterate over all the frames in a recording

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
recordings = Recordings(connection)
frames = Frames(connection)

cursor = recordings.all()

# Get the first recording
recording = Recording(cursor)
print(" ", recording.id, " ", recording.directory)

if recording is None:
    print("No recordings!")
    exit()

# Get all frames of a recording
cursor = frames.query(recordingid=recording.id)

frame = Frame(cursor)

while frame is not None:
    print(" ", frame.recordingid, " ", frame.uuid, " ", frame.longitude, " ", frame.latitude)
    frame = Frame(cursor)


# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Frame
from horus_media import Client, ImageRequestBuilder, ImageProvider, Mode, Scales, Direction

# This example shows how to request a panoramic image

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
frames = Frames(connection)
client = Client()
image_provider = ImageProvider()

# Get a frame
frame = Frame(frames.query())

if frame is None:
    print("No recordings!")
    exit()

# Set parameters
mode = Mode.panoramic
scale = Scales.Px_2048

# Get the image
request_builder = ImageRequestBuilder(frame.recordingid, frame.uuid)
request = client.fetch(request_builder.build(mode, scale))
result = image_provider.fetch(request)

# Save the file
filename = ".\\panoramic_{}.jpg".format(frame.index)

with open(filename, 'wb') as f:
    f.write(result.image.getvalue())
    result.image.close()



# Copyright(C) 2021 Horus View and Explore B.V.
"""Horus Media Server strategy example

Example utility
"""

import psycopg2
import argparse
import logging

from horus_media import Client, Geometry


def create_argument_parser():
    return argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)


def add_database_arguments(parser):
    # db
    parser.add_argument("--db-name", type=str,
                        default="HorusWebMoviePlayer", help="the database name")
    parser.add_argument("--db-user", type=str, default="postgres",
                        help="database user name used to authenticate")
    parser.add_argument("--db-password", type=str,
                        help="database password used to authenticate")
    parser.add_argument("--db-host", type=str, default="localhost",
                        help="database database host address")
    parser.add_argument("--db-port", type=int, default=5432,
                        help="database connection port number")


def add_server_arguments(parser):
    parser.add_argument("-s", "--server",  metavar="URL", type=str, default="http://localhost:5050/web/",
                        help="Horus Media Server endpoint")


def add_size_argument(parser, default=(1024, 1024)):
    parser.add_argument("-S", "--size",  metavar=("WIDTH", "HEIGHT"), nargs=2, type=int, default=default,
                        help="size of the image")


def add_geometry_arguments(parser):
    parser.add_argument("-gsc", "--geom-scale", type=int, default=400,
                        help="output scale in px/m (orthographic)")
    parser.add_argument("-gw", "--geom-width", type=float, default=6,
                        help="geometry width (orthographic)")
    parser.add_argument("-gh", "--geom-height", type=float, default=2,
                        help="geometry height (orthographic)")
    parser.add_argument("-gd", "--geom-dist", type=float, default=4,
                        help="geometry distance (orthographic)")
    parser.add_argument("-gs", "--geom-shift", type=float, default=0,
                        help="geometry shift (orthographic)")

def get_database_connection_string(args):
    db_params = [("host", args.db_host),
                 ("port", str(args.db_port)),
                 ("dbname", args.db_name),
                 ("user", args.db_user),
                 ("password", args.db_password),
                 ]

    return " ".join(
        map("=".join, filter(lambda x: x[1] != None, db_params)))


def get_connection(args):
    try:
        return psycopg2.connect(get_database_connection_string(args))
    except psycopg2.OperationalError as exception:
        logging.error(f"{exception} Connecting to database")
        exit()


def get_geometry(args, altitude_next = None):
    return Geometry(args.geom_scale, args.geom_width,
                    args.geom_height, args.geom_dist, args.geom_shift, altitude_next)


def get_client(args):
    try:
        return Client(args.server)
    except OSError as exception:
        logging.error(f"{exception}. Connecting to server {args.server}")
        exit()



# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Frame

# This example shows how to geo-query frames

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
frames = Frames(connection)

# Get a frame
frame = Frame(frames.query())

if frame is None:
    print("No recordings!")
    exit()

# Get frames within a certain distance of this frame
point = (frame.longitude, frame.latitude)  # EPSG:4326 (lon, lat)
distance = 20  # in meters
cursor = frames.query(within=(point, distance))

frame = Frame(cursor)

while frame is not None:
    print(" ", frame.uuid, " ", frame.longitude, " ", frame.latitude)
    frame = Frame(cursor)


# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2
import numpy
import cv2
from typing import NamedTuple
from matplotlib import patches as patches, pyplot as pyplot

from horus_db import Recordings, Recording, Frames, Frame
from horus_media import Client, ImageRequestBuilder, ImageProvider, ComputationRequestBuilder, \
                                ComputationProvider, Size, Direction
from horus_gis import PositionVector, Geographic

# This example shows how to use triangulation to get geographic positions 
# on boxes drawn by the user on 3 consecutive frames

connection = psycopg2.connect("dbname=HorusWebMoviePlayer user=postgres password=horusweb")
client = Client()
image_provider = ImageProvider()
computation_provider = ComputationProvider()

def main():
    no_of_frames = 3
    no_of_points = 4

    # Example parameters
    size = Size(718, 476)
    direction = Direction(0, 0)
    hor_fov = 112.915016

    # Get a frame, see 'get_frame.py' to get a specific frame
    frame = get_frame()

    if frame is None:
        print("No recordings!")
        exit()

    pos_vecs = []

    # Request image, apply detection and request position vector for detection 
    for i in range(no_of_frames):
        # Get the image and apply detection
        request = get_image_request(frame, size, direction, hor_fov)
        box = get_detection(image_provider.fetch(request).image.getvalue())

        # Request position vector for each pixel coordinate
        for pix_coor in box:
            for px in pix_coor:
                pos_vecs.append(get_position_vector(frame, request, px))

        frame = get_next_frame(frame)


    # Triangulate position for each box corner
    positions = []
    for i in range(no_of_points):
        positions.append(Geographic.triangulate([pos_vecs[i], pos_vecs[i + no_of_points], 
                                        pos_vecs[i + (2 * no_of_points)]]))

    plot_results(pos_vecs, positions)


class PixelCoordinate(NamedTuple):
    x: int
    y: int

class BoundingBox(NamedTuple):
    px: PixelCoordinate

def get_frame():
    recordings = Recordings(connection)
    frames = Frames(connection)
    cursor = recordings.all()
    recording = Recording(cursor)
    cursor = frames.query(recordingid=recording.id, index=0)
    return Frame(cursor)

def get_next_frame(frame):
    frames = Frames(connection)
    cursor = frames.query(recordingid=frame.recordingid, index=frame.index + 1)
    return Frame(cursor)

def get_image_request(frame, size, direction, hor_fov):
    request_builder = ImageRequestBuilder(frame.recordingid, frame.uuid)
    return client.fetch(request_builder.build_spherical(size, direction, hor_fov))

def get_position_vector(frame, image_request, px):
    request_builder = ComputationRequestBuilder(frame.recordingid, frame.uuid)
    request = client.fetch(request_builder.build(image_request.size, image_request.direction, 
                                                 image_request.fov, px.x, px.y))
    result = computation_provider.fetch(request)
    return PositionVector(*result.values())

def get_detection(image):
    window_name = 'Draw a box, then escape to exit'
    ix = -1
    iy = -1
    global is_drawing
    global box

    buffer = numpy.frombuffer(image, dtype='uint8')
    img = cv2.imdecode(buffer, cv2.IMREAD_COLOR)
    is_drawing = False

    def draw_rectangle(event, x, y, flags, param):
        global is_drawing, ix, iy, box
        if event == cv2.EVENT_LBUTTONDOWN:
            is_drawing = True
            ix = x
            iy = y
        elif event == cv2.EVENT_LBUTTONUP:
            is_drawing = False
            cv2.rectangle(img, (ix, iy), (x, y), (0, 255, 0), -1)
            box = BoundingBox([PixelCoordinate(ix, iy), PixelCoordinate(x, iy),
            PixelCoordinate(x, y), PixelCoordinate(ix, y)])
        elif event == cv2.EVENT_MOUSEMOVE:
            if is_drawing:
                cv2.rectangle(img, (ix, iy), (x, y), (0, 255, 0), -1)

    cv2.namedWindow(window_name)
    cv2.setMouseCallback(window_name, draw_rectangle)

    while True:
        cv2.imshow(window_name, img)

        if cv2.waitKey(5) & 0xFF == 27:
            break

    cv2.destroyAllWindows()

    return box

def plot_results(pos_vecs, positions):
    camera_positions = numpy.array(pos_vecs)
    detection_positions = numpy.array(positions)
    pyplot.plot(camera_positions[:,1], camera_positions[:,0], 'ro')
    pyplot.plot(detection_positions[:,1], detection_positions[:,0], 'bo')

    pyplot.legend(handles=[patches.Patch(color='red', label='camera'), 
        patches.Patch(color='blue', label='detection')], loc='lower right')
    pyplot.ticklabel_format(useOffset=False, style='plain')
    pyplot.title('Horus Triangulation Example')
    pyplot.ylabel('Lat')
    pyplot.xlabel('Lon')
    pyplot.show()

main()



# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Recordings, Recording

# This example shows how to get a specific recording

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
recordings = Recordings(connection)

cursor = recordings.all()

# Get the first recording
recording = Recording(cursor)

if recording is None:
    print("No recordings!")
    exit()

# Alternatively select a recording by id
cursor = recordings.get(id=recording.id)
recording = Recording(cursor)

# Or query the database for a recording by directory
cursor = connection.cursor()
cursor.execute("""SELECT id, recordingdirectory, boundingbox, fileformat
FROM recordings WHERE recordingdirectory = %s;""", (recording.directory,))
recording = Recording(cursor)

print(" ", recording.id, " ", recording.directory)


# Copyright(C) 2020 Horus View and Explore B.V.

import psycopg2

from horus_db import Frames, Frame

# This example shows how to get a specific frame

def get_connection():
    return psycopg2.connect(
        "dbname=HorusWebMoviePlayer user=postgres password=horusweb")

connection = get_connection()
frames = Frames(connection)

# Get a frame
frame = Frame(frames.query())

if frame is None:
    print("No recordings!")
    exit()

# Get frame by its uuid
cursor = frames.query(guid=frame.uuid)

frame = Frame(cursor)
print(" ", frame.uuid, " ", frame.longitude, " ", frame.latitude)

# Alternatively get frame by recording id and index (next frame)
cursor = frames.query(recordingid=frame.recordingid, index=frame.index + 1)

if frame is None:
    print("No next frame!")
    exit()

frame = Frame(cursor)
print(" ", frame.uuid, " ", frame.longitude, " ", frame.latitude)